# ruff: noqa: D100, D102, D103, S101, S106

import pytest
from httpx import AsyncClient
from httpx import HTTPError
from pytest_httpx import HTTPXMock

from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import NetworkError


@pytest.mark.asyncio
async def test_create_app(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        json={
            "client_id": "client-id",
            "client_secret": "client-secret",
        },
        headers={},
    )
    client = AsyncClient()
    client_id, client_secret = await ActivityPub.create_app(
        client=client,
        instance_url="https://instance.url",
    )

    request = httpx_mock.get_request()

    assert request.url == "https://instance.url/api/v1/apps"
    assert client_id == "client-id"
    assert client_secret == "client-secret"  # noqa: S105


@pytest.mark.asyncio
async def test_create_app_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    with pytest.raises(NetworkError):
        _token = await ActivityPub.create_app(
            client=client,
            instance_url="instance.url",
        )



# ruff: noqa: D100, D102, D103, S101, S106

import pytest
from httpx import AsyncClient
from httpx import HTTPError
from pytest_httpx import HTTPXMock

from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import NetworkError


@pytest.mark.asyncio
async def test_get_account_statuses(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        json=[
            {
                "id": "108880211901672326",
                "created_at": "2022-08-24T22:29:46.493Z",
                "in_reply_to_id": "108880209317577809",
                "in_reply_to_account_id": "103641",
                "sensitive": "false",
            },
        ],
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    response = await instance.get_account_statuses(account_id="acc-id", max_id="max-id", min_id="min-id")

    request = httpx_mock.get_request()

    assert request.url == "https://instance.url/api/v1/accounts/acc-id/statuses?max_id=max-id&min_id=min-id"
    assert response[0].get("id") == "108880211901672326"


@pytest.mark.asyncio
async def test_get_account_statuses_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(NetworkError):
        await instance.get_account_statuses(account_id="123456789")



# ruff: noqa: D100, D102, D103, S101, S106
from typing import Any

import pytest
from httpx import AsyncClient
from httpx import HTTPError
from pytest_httpx import HTTPXMock

from minimal_activitypub.client_2_server import INSTANCE_TYPE_PLEROMA
from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import NetworkError


@pytest.fixture
def status() -> Any:  # nocl
    return {
        "id": "103254193998341330",
        "created_at": "2019-12-05T08:19:26.052Z",
        "in_reply_to_id": "null",
        "in_reply_to_account_id": "null",
        "sensitive": "false",
        "spoiler_text": "",
        "visibility": "public",
        "language": "en",
        "uri": "https://mastodon.social/users/trwnh/statuses/103254193998341330",
        "url": "https://mastodon.social/@trwnh/103254193998341330",
        "replies_count": 0,
        "reblogs_count": 0,
        "favourites_count": 0,
        "favourited": False,
        "reblogged": False,
        "muted": "false",
        "bookmarked": "false",
        "pinned": "false",
        "text": "test",
        "reblog": "null",
        "application": {"name": "Web", "website": "null"},
        "account": {
            "id": "14715",
            "username": "trwnh",
            "acct": "trwnh",
            "display_name": "infinite love ⴳ",
        },
        "media_attachments": [
            {
                "id": "22345792",
                "type": "image",
                "url": "https://files.mastodon.social/media_attachments/files/022/345/792/original/57859aede991da25.jpeg",
                "preview_url": "https://files.mastodon.social/media_attachments/files/022/345/792/small/57859aede991da25.jpeg",
                "remote_url": "null",
                "text_url": "https://mastodon.social/media/2N4uvkuUtPVrkZGysms",
                "meta": {
                    "original": {"width": 640, "height": 480, "size": "640x480", "aspect": 1.3333333333333333},
                    "small": {"width": 461, "height": 346, "size": "461x346", "aspect": 1.3323699421965318},
                    "focus": {"x": -0.27, "y": 0.51},
                },
                "description": "test media description",
                "blurhash": "UFBWY:8_0Jxv4mx]t8t64.%M-:IUWGWAt6M}",
            }
        ],
        "mentions": [],
        "tags": [],
        "emojis": [],
        "card": "null",
        "poll": "null",
    }


@pytest.mark.asyncio
async def test_delete_status_id(httpx_mock: HTTPXMock, status) -> None:
    httpx_mock.add_response(
        json=status,
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    response = await instance.delete_status(status="1234")

    request = httpx_mock.get_request()

    assert request.url == "https://instance.url/api/v1/statuses/1234"
    assert request.method == "DELETE"
    assert response == status


@pytest.mark.asyncio
async def test_delete_status_status(httpx_mock: HTTPXMock, status) -> None:
    httpx_mock.add_response(
        json=status,
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    response = await instance.delete_status(status=status)

    request = httpx_mock.get_request()

    assert request.url == "https://instance.url/api/v1/statuses/103254193998341330"
    assert request.method == "DELETE"
    assert response == status


@pytest.mark.asyncio
async def test_delete_status_reblogged(httpx_mock: HTTPXMock, status) -> None:
    httpx_mock.add_response(
        json=status,
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    instance.instance_type = INSTANCE_TYPE_PLEROMA
    status["reblogged"] = True
    status["reblog"] = {"id": "123456789"}

    response = await instance.delete_status(status=status)

    request = httpx_mock.get_request()

    assert request.url == "https://instance.url/api/v1/statuses/123456789/unreblog"
    assert request.method == "POST"
    assert response.get("id") == "103254193998341330"


@pytest.mark.asyncio
async def test_delete_status_favourited(httpx_mock: HTTPXMock, status) -> None:
    httpx_mock.add_response(
        json=status,
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    instance.instance_type = INSTANCE_TYPE_PLEROMA
    status["favourited"] = True

    response = await instance.delete_status(status=status)

    request = httpx_mock.get_request()

    assert request.url == "https://instance.url/api/v1/statuses/103254193998341330/unfavourite"
    assert request.method == "POST"
    assert response.get("id") == "103254193998341330"


@pytest.mark.asyncio
async def test_undo_reblog_status_id(httpx_mock: HTTPXMock, status) -> None:
    httpx_mock.add_response(
        json=status,
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    instance.instance_type = INSTANCE_TYPE_PLEROMA
    status["reblogged"] = True

    response = await instance.undo_reblog(status="103254193998341330")

    request = httpx_mock.get_request()

    assert request.url == "https://instance.url/api/v1/statuses/103254193998341330/unreblog"
    assert request.method == "POST"
    assert response.get("id") == "103254193998341330"


@pytest.mark.asyncio
async def test_delete_status_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(NetworkError):
        await instance.delete_status(status="111")


@pytest.mark.asyncio
async def test_undo_reblog_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    instance.instance_type = INSTANCE_TYPE_PLEROMA

    with pytest.raises(NetworkError):
        await instance.undo_reblog(status="111")


@pytest.mark.asyncio
async def test_undo_favourite_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    instance.instance_type = INSTANCE_TYPE_PLEROMA

    with pytest.raises(NetworkError):
        await instance.undo_favourite(status="111")



# ruff: noqa: D100, D102, D103, S101, S106
from pathlib import Path

import pytest
from httpx import AsyncClient
from httpx import HTTPError
from pytest_httpx import HTTPXMock

from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import NetworkError


@pytest.mark.asyncio
@pytest.mark.httpx_mock(assert_all_responses_were_requested=False)
async def test_post_media(httpx_mock: HTTPXMock) -> None:
    expected_response = {
        "id": "22348641",
        "type": "image",
        "url": "https://files.mastodon.social/media_attachments/files/022/348/641/original/e96382f26c72a29c.jpeg",
        "preview_url": "https://files.mastodon.social/media_attachments/files/022/348/641/small/e96382f26c72a29c.jpeg",
        "remote_url": None,
        "text_url": "https://mastodon.social/media/4Zj6ewxzzzDi0g8JnZQ",
        "meta": {
            "focus": {"x": -0.42, "y": 0.69},
            "original": {"width": 640, "height": 480, "size": "640x480", "aspect": 1.3333333333333333},
            "small": {"width": 461, "height": 346, "size": "461x346", "aspect": 1.3323699421965318},
        },
        "description": "test uploaded via api",
        "blurhash": "UFBWY:8_0Jxv4mx]t8t64.%M-:IUWGWAt6M}",
    }

    httpx_mock.add_response(
        json=expected_response,
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    Path("/tmp/dummy").touch(exist_ok=True)  # noqa: S108
    dummy_file = Path("/tmp/dummy").open(mode="rb")  # noqa: S108
    response = await instance.post_media(
        file=dummy_file,
        mime_type="jpg",
        description="Test Description",
        focus=(0.1, 0.5),
    )
    request = httpx_mock.get_request()

    dummy_file.close()

    assert request.url == "https://instance.url/api/v1/media"
    assert request.method == "POST"
    assert response == expected_response


@pytest.mark.asyncio
@pytest.mark.httpx_mock(assert_all_responses_were_requested=False)
async def test_post_media_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    Path("/tmp/dummy").touch(exist_ok=True)  # noqa: S108
    dummy_file = Path("/tmp/dummy").open(mode="rb")  # noqa: S108
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(NetworkError):
        await instance.post_media(file=dummy_file, mime_type="jpg")

    dummy_file.close()



"""Package 'minimal_activitypub' level definitions."""

import sys
from enum import Enum
from importlib.metadata import version
from typing import Any
from typing import Dict
from typing import Final

__display_name__: Final[str] = "Minimal-ActivityPub"
__version__: Final[str] = version(__package__)

USER_AGENT: Final[str] = f"{__display_name__}_v{__version__}_Python_{sys.version.split()[0]}"

Status = Dict[str, Any]


class Visibility(str, Enum):
    """Enumerating possible Visibility values for statuses."""

    PUBLIC = "public"
    UNLISTED = "unlisted"
    PRIVATE = "private"
    DIRECT = "direct"


class SearchType(str, Enum):
    """Enumerating possible type values for status searches."""

    ACCOUNTS = "accounts"
    HASHTAGS = "hashtags"
    STATUSES = "statuses"



# ruff: noqa: D100, D102, D103, S101, S106

import pytest
from httpx import AsyncClient
from httpx import HTTPError
from pytest_httpx import HTTPXMock

from minimal_activitypub.client_2_server import INSTANCE_TYPE_PLEROMA
from minimal_activitypub.client_2_server import INSTANCE_TYPE_TAKAHE
from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import NetworkError


@pytest.mark.asyncio
async def test_determine_instance_type_takahe(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        json={
            "version": "takahe v12.3.4",
            "configuration": {
                "statuses": {
                    "max_characters": 500,
                    "max_media_attachments": 10,
                },
                "media_attachments": {
                    "image_size_limit": 1234567,
                    "supported_mime_types": [
                        "jpg",
                        "webp",
                        "mp4",
                    ],
                },
            },
        },
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="instance.url",
        client=client,
        access_token="access_token",
    )
    await instance.determine_instance_type()

    assert instance.instance_type == INSTANCE_TYPE_TAKAHE
    assert instance.max_status_len == 500
    assert instance.max_attachments == 10
    assert instance.max_att_size == 1234567
    assert "jpg" in instance.supported_mime_types
    assert "webp" in instance.supported_mime_types
    assert "mp4" in instance.supported_mime_types
    assert "mkv" not in instance.supported_mime_types


@pytest.mark.asyncio
async def test_determine_instance_type_pleroma(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        json={
            "version": "Pleroma v37.8.9",
            "max_characters": 1000,
            "max_media_attachments": 20,
            "configuration": {
                "media_attachments": {
                    "image_size_limit": 2345678,
                    "supported_mime_types": [
                        "mkv",
                    ],
                },
            },
        },
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="instance.url",
        client=client,
        access_token="access_token",
    )
    await instance.determine_instance_type()

    assert instance.instance_type == INSTANCE_TYPE_PLEROMA
    assert instance.max_status_len == 1000
    assert instance.max_attachments == 20
    assert instance.max_att_size == 2345678
    assert instance.supported_mime_types is not None
    assert "jpg" not in instance.supported_mime_types
    assert "webp" not in instance.supported_mime_types
    assert "mp4" not in instance.supported_mime_types
    assert "mkv" in instance.supported_mime_types


@pytest.mark.asyncio
async def test_verify_creds_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(NetworkError):
        await instance.determine_instance_type()



# ruff: noqa: D100, D102, D103, S101, S106

from httpx import AsyncClient

from minimal_activitypub.client_2_server import ActivityPub


def test_parse_next_prev_none() -> None:
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )

    instance._parse_next_prev(links=None)

    assert instance.pagination == {
        "next": {"max_id": None, "min_id": None},
        "prev": {"max_id": None, "min_id": None},
    }


def test_parse_next_prev() -> None:
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    links = (
        "Link: "
        '<https://mastodon.example/api/v1/endpoint?max_id=7163058>; rel="next", '
        '<https://mastodon.example/api/v1/endpoint?min_id=7275607>; rel="prev"'
    )

    instance._parse_next_prev(links=links)

    assert isinstance(instance.pagination, dict)
    assert instance.pagination["next"]["min_id"] is None
    assert instance.pagination["next"]["max_id"] == "7163058"
    assert instance.pagination["prev"]["min_id"] == "7275607"
    assert instance.pagination["prev"]["max_id"] is None



# ruff: noqa: D100, D102, D103, S101, S106

import pytest
from httpx import AsyncClient
from httpx import HTTPError
from pytest_httpx import HTTPXMock

from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import NetworkError


@pytest.mark.asyncio
async def test_validate_authorization_code(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        json={"access_token": "acc-token"},
        headers={},
    )
    client = AsyncClient()
    response = await ActivityPub.validate_authorization_code(
        client=client,
        instance_url="https://instance.url",
        authorization_code="123456",
        client_id="client-id",
        client_secret="client-secret",
    )

    request = httpx_mock.get_request()

    assert request.url == "https://instance.url/oauth/token"
    assert response == "acc-token"


@pytest.mark.asyncio
async def test_validate_authorization_code_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    with pytest.raises(NetworkError):
        _token = await ActivityPub.validate_authorization_code(
            client=client,
            instance_url="instance.url",
            authorization_code="123456",
            client_id="client-id",
            client_secret="client-secret",
        )



# ruff: noqa: D100, D102, D103, S101, S106

import pytest
from httpx import AsyncClient
from httpx import HTTPError
from pytest_httpx import HTTPXMock

from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import NetworkError
from minimal_activitypub.client_2_server import ServerError


@pytest.mark.asyncio
async def test_verify_creds_ok(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(json={}, headers={})
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    response = await instance.verify_credentials()
    assert response is not None


@pytest.mark.asyncio
async def test_verify_creds_server_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        status_code=500,
        json={"error": "System Error Occurred"},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(ServerError):
        _response = await instance.verify_credentials()


@pytest.mark.asyncio
async def test_verify_creds_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(NetworkError):
        _response = await instance.verify_credentials()



# ruff: noqa: D100, D102, D103, S101

import pytest

from minimal_activitypub.client_2_server import ActivityPub


@pytest.mark.asyncio
async def test_generate_auth_url() -> None:
    url = await ActivityPub.generate_authorization_url(
        instance_url="instance.url",
        client_id="client_id",
        user_agent="user_agent",
    )

    assert (
        url
        == "https://instance.url/oauth/authorize?response_type=code&client_id=client_id&redirect_uri=urn%3Aietf%3Awg%3Aoauth%3A2.0%3Aoob&scope=read+write"
    )



# ruff: noqa: D100, D102, D103, S101, S106

import pytest
from httpx import AsyncClient
from httpx import HTTPError
from pytest_httpx import HTTPXMock

from minimal_activitypub import SearchType
from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import NetworkError


@pytest.mark.asyncio
async def test_search(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        json={
            "accounts": [],
            "hashtags": [],
            "statuses": [
                {
                    "id": "108880211901672326",
                    "created_at": "2022-08-24T22:29:46.493Z",
                    "in_reply_to_id": "108880209317577809",
                    "in_reply_to_account_id": "103641",
                    "sensitive": "false",
                },
            ],
        },
        headers={},
    )
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    response = await instance.search(
        query="abc",
        query_type=SearchType.STATUSES,
        account_id="akjsghfa",
        max_id="max-id",
        min_id="min-id",
        limit=12,
        offset=5,
        following=True,
        exclude_unreviewed=True,
    )

    request = httpx_mock.get_request()

    assert request.url == (
        "https://instance.url/api/v2/search?q=abc&type=statuses&resolve=true&following=true&account_id=akjsghfa"
        "&exclude_unreviewed=true&max_id=max-id&min_id=min-id&limit=12&offset=5"
    )
    assert response["statuses"][0].get("id") == "108880211901672326"


@pytest.mark.asyncio
async def test_get_public_timeline_network_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_exception(HTTPError("Unable to read within timeout"))
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(NetworkError):
        await instance.search(query="cats", query_type=SearchType.STATUSES)



# ruff: noqa: D100, D102, D103, S101, S106
from datetime import datetime
from datetime import timedelta

import pytest
from httpx import AsyncClient

from minimal_activitypub.client_2_server import UTC
from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import RatelimitError


@pytest.mark.asyncio
async def test_update_ratelimit() -> None:
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    headers = {
        "X-RateLimit-Limit": "300",
        "X-RateLimit-Remaining": "0",
        "X-RateLimit-Reset": (datetime.now(tz=UTC) + timedelta(minutes=5)).isoformat(),
    }

    instance._update_ratelimit(headers=headers)

    assert instance.ratelimit_limit == 300
    assert instance.ratelimit_remaining == 0
    assert isinstance(instance.ratelimit_reset, datetime)

    with pytest.raises(RatelimitError):
        await instance._pre_call_checks()



# ruff: noqa: D100, D102, D103, S101, S106

import pytest
from httpx import AsyncClient
from pytest_httpx import HTTPXMock

from minimal_activitypub.client_2_server import ActivityPub
from minimal_activitypub.client_2_server import ClientError
from minimal_activitypub.client_2_server import ConflictError
from minimal_activitypub.client_2_server import ForbiddenError
from minimal_activitypub.client_2_server import GoneError
from minimal_activitypub.client_2_server import NotFoundError
from minimal_activitypub.client_2_server import RatelimitError
from minimal_activitypub.client_2_server import UnauthorizedError
from minimal_activitypub.client_2_server import UnprocessedError


@pytest.mark.asyncio
async def test_unauthorized_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(status_code=401)
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(UnauthorizedError):
        await instance.get_public_timeline()


@pytest.mark.asyncio
async def test_forbidden_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(status_code=403)
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(ForbiddenError):
        await instance.get_public_timeline()


@pytest.mark.asyncio
async def test_not_found_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(status_code=404)
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(NotFoundError):
        await instance.get_public_timeline()


@pytest.mark.asyncio
async def test_conflict_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(status_code=409)
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(ConflictError):
        await instance.get_public_timeline()


@pytest.mark.asyncio
async def test_gone_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(status_code=410)
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(GoneError):
        await instance.get_public_timeline()


@pytest.mark.asyncio
async def test_unprocessed_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(status_code=422)
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(UnprocessedError):
        await instance.get_public_timeline()


@pytest.mark.asyncio
async def test_rate_limit_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(status_code=429)
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(RatelimitError):
        await instance.get_public_timeline()


@pytest.mark.asyncio
async def test_client_error(httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(status_code=444)
    client = AsyncClient()
    instance = ActivityPub(
        instance="https://instance.url",
        client=client,
        access_token="access_token",
    )
    with pytest.raises(ClientError):
        await instance.get_public_timeline()
